@@ -942,6 +942,12 @@ class Cursor(object):
 
             for doc in collection.find():
                 print(doc)
+
+        .. note:: Even if :attr:`alive` is True, :meth:`next` can raise
+          :exc:`StopIteration`. :attr:`alive` can also be True while iterating
+          a cursor from a failed server. In this case :attr:`alive` will
+          return False after :meth:`next` fails to retrieve the next batch
+          of results from the server.
         """
         return bool(len(self.__data) or (not self.__killed))
 
@@ -969,7 +975,8 @@ class Cursor(object):
     def __iter__(self):
         return self
 
-    def __next__(self):
+    def next(self):
+        """Advance the cursor."""
         if self.__empty:
             raise StopIteration
         _db = self.__collection.database
@@ -982,7 +989,7 @@ class Cursor(object):
         else:
             raise StopIteration
 
-    next = __next__
+    __next__ = next
 
     def __enter__(self):
         return self
